@@ -31,6 +31,7 @@
 #include <signal.h>
 #include <unistd.h>
 #include <stdlib.h>
+#include <sys/socket.h>
 #include <libbonobo.h>
 #include <netdb.h>
 #include "festivalsynthesisdriver.h"
@@ -622,8 +623,8 @@ festival__get_synthesizerVersion (PortableServer_Servant servant,
 static void
 voice_list_free (GSList *l)
 {
-	g_assert (l);
 	GSList *tmp = l;
+	g_assert (l);
 	
 	while (tmp)
 	{
@@ -646,8 +647,8 @@ get_voice_list_with_spec (GSList *l,
 	tmp = l;
 	while (tmp) {
 		i = (GNOME_Speech_VoiceInfo *) tmp->data;
-		if ((strlen(info->name) ? strcmp (i->name, info->name) : 1) && 
-		    (strlen(info->language) ? strcmp (i->language, info->language) : 1)) {
+		if ((strlen(info->name) ? !strcmp (i->name, info->name) : 1) && 
+		    (strlen(info->language) ? !strcmp (i->language, info->language) : 1)) {
 			new_info = GNOME_Speech_VoiceInfo__alloc ();
             		new_info->language = CORBA_string_dup(info->language);
             		new_info->name = CORBA_string_dup (info->name);
